Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;

USE work.pkg_081006.all;

----------------------------------------------------
-- This module provides timing and control for the
-- Booster BLM Integrator control.

entity DAQ_Control is
port(
		clk			: in std_logic;
		nreset		: in std_logic;
		
		SW5			: in std_logic; -- on board DIP switches
		SW6			: in std_logic;
		SW7			: in std_logic;
		
		T1			: in std_logic; -- Front panel TTL Trigger 1
		VME_Start	: in std_logic; -- VME Start DAQ trigger

		Run_DAQ		: out std_logic;
		Clr_DAQ		: out std_logic;

		Use_Const 	: out std_logic;  
		Disable_Baseline : out std_logic
	);
end DAQ_Control;

--########################################################
--				A R C H I T E C T U R E
--########################################################
architecture main of DAQ_Control is

-- Synchronized Input Signals --------------------------------
	signal trg1		: std_logic;
	signal vstrt	: std_logic;
	
	signal sw		: std_logic_vector(7 downto 5);
	
	signal start	: std_logic;
	signal stop		: std_logic;

-- Booster Cycle Timer ---------------------------------------
	signal btime		: std_logic_vector(19 downto 0);
	signal bcycle		: std_logic;
	
-- SETUP_STATE_MACHINE ---------------------------------------
	type STATE_TYPE is
	(idle, init_daq, daq_on, daq_off);
	signal CS, NS : STATE_TYPE;

--========================================================
--						B E G I N
--========================================================
begin

SYNC_T1: led 
	port map(sig_in=>T1, clk=>clk,	clrn=>nreset, pls_out=>trg1); 
SYNC_V1: led 
	port map(sig_in=>VME_Start, clk=>clk,	clrn=>nreset, pls_out=>vstrt); 

SYNC_OPTION_SELECT: 
process(clk, nreset, SW5, SW6, SW7)
begin
	if(nreset='0')			then 
		sw <= "000";
	elsif(rising_edge(clk))	then 
		sw <= SW7 & SW6 & SW5;
	end if;

end process;

-----------------------------------------------------------
-- The signal "bcycle" will be active for 40ms after the start
-- trigger.

BOOSTER_CYCLE_TIMER:
process(clk,nreset,btime,bcycle)
begin
	if(nreset ='0')			then 
		btime <= (others => '0');
		
	elsif(rising_edge(clk))	then 
		if   (start = '1')		then 
			btime  <= X"F4240";
			bcycle <= '1';
		elsif(btime > X"00000")	then 
			btime  <= btime - 1;
			bcycle <= '1';
		else						 
			btime  <= btime;
			bcycle <= '0';
		end if;
	end if;
	stop <= not bcycle;
	
end process;

--@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
-- 				O P T I O N   S E L E C T I O N
--@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
OPTION_SELECTION:
process(sw, trg1, vstrt)
begin
	---------------------- defaults --
	start     			<= trg1;
	Use_Const 			<= '0';
	Disable_Baseline 	<= '0';
	----------------------------------
	
	if (sw(7) = '1') then -- VME Control, SW7 = HIGH
		start      <= vstrt;
	end if;
	
	if (sw(6) = '0') then -- Disable Baseline Meas., SW6 = LOW
		Disable_Baseline <= '1';
	end if;
	
	if (sw(5) = '1') then -- Use Test Vectors, SW5 = HIGH
		Use_Const <= '1';
		Disable_Baseline <= '1';
	end if;
end process;	
		
--{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{{
--                S T A T E   M A C H I N E
--}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}}
SM_REG: process(clk, nreset)
begin
	if(nreset='0')			then CS <= idle;
	elsif(rising_edge(clk))	then CS <= NS;
	end if;
end process;

SM_NEXT_STATE: 
process(CS, start, stop)
begin
	Run_DAQ   <= '1';
	Clr_DAQ	  <= '0';
	
	case CS is
		when idle =>					NS <= init_daq;
			Run_DAQ   <= '0';
			
		when init_daq =>				NS <= daq_on;
			Run_DAQ   <= '0';
			Clr_DAQ	  <= '1';

		when daq_on => 					
			if   (stop = '1') then		NS <= daq_off;
			elsif(start = '1')then 	    NS <= init_daq;
			else						NS <= daq_on;
			end if;
		
		when daq_off => 					
			Run_DAQ   <= '0';
			if   (start = '1')then 	    NS <= init_daq;
			else						NS <= daq_off;
			end if;

		when others => 					NS <= idle;
	end case;
end process;

end;

